# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from ...v5_1.test_plan.models import *
from .test_plan_client import TestPlanClient

__all__ = [
    'BuildDefinitionReference',
    'CloneOperationCommonResponse',
    'CloneOptions',
    'CloneStatistics',
    'CloneTestPlanOperationInformation',
    'CloneTestPlanParams',
    'CloneTestSuiteOperationInformation',
    'CloneTestSuiteParams',
    'Configuration',
    'DestinationTestPlanCloneParams',
    'DestinationTestSuiteInfo',
    'GraphSubjectBase',
    'IdentityRef',
    'LastResultDetails',
    'NameValuePair',
    'PointAssignment',
    'ReferenceLinks',
    'ReleaseEnvironmentDefinitionReference',
    'Results',
    'SourceTestPlanInfo',
    'SourceTestplanResponse',
    'SourceTestSuiteInfo',
    'SuiteEntry',
    'SuiteEntryUpdateParams',
    'SuiteTestCaseCreateUpdateParameters',
    'TeamProjectReference',
    'TestCase',
    'TestCaseReference',
    'TestConfiguration',
    'TestConfigurationCreateUpdateParameters',
    'TestConfigurationReference',
    'TestEnvironment',
    'TestOutcomeSettings',
    'TestPlan',
    'TestPlanCreateParams',
    'TestPlanDetailedReference',
    'TestPlanReference',
    'TestPlansHubRefreshData',
    'TestPlanUpdateParams',
    'TestPoint',
    'TestPointCount',
    'TestPointResults',
    'TestPointUpdateParams',
    'TestSettings',
    'TestSuite',
    'TestSuiteCreateParams',
    'TestSuiteCreateUpdateCommonParams',
    'TestSuiteReference',
    'TestSuiteReferenceWithProject',
    'TestSuiteUpdateParams',
    'TestVariable',
    'TestVariableCreateUpdateParameters',
    'WorkItem',
    'WorkItemDetails',
    'TestPlanClient'
]
